@@ -338,8 +338,8 @@ class Config:
     def github_issue(title, body):
         url = f"https://github.com/CZ-NIC/convey/issues/new?title={quote(title)}&body={quote(body)}"
         if len(url) > 2000:
-            # strip possible ending percent (because double percent would not work)
-            url = url[:1987].rstrip("%") + "%7D%60%60%60"  # add newline and 3× backtick
+            # strip ending url-encoded character (because things like "%2" instead of "%20" would break the URL)
+            url = re.sub(r'%(\d{1,2})?$', '', url[:1987]) + "%7D%60%60%60"  # add newline and 3× backtick
         webbrowser.open(url)
         input(f"\nPlease submit a Github issue at {url}"
               "\nTrying to open issue tracker in a browser...")
