@@ -4,18 +4,18 @@ import re
 import smtplib
 import sys
 from abc import abstractmethod, ABC
-from email.utils import getaddresses
 from socket import gaierror
 
-from envelope import envelope
 from validate_email import validate_email
 
+from envelope import Envelope
 from .attachment import Attachment
 from .config import Config
 
 re_title = re.compile('<title>([^<]*)</title>')
 logger = logging.getLogger(__name__)
 
+OTRS_VERSION = 6
 
 class MailSender(ABC):
     def __init__(self, parser):
@@ -32,16 +32,18 @@ class MailSender(ABC):
         pass
 
     def send_list(self, mails):
-        """ Send a bunch of e-mail messages """
+        """ Send a bunch of e-mail messages 
+        :type mails: Union[list, generator]
+        """
         sent_mails = 0
         total_count = 0
         status = None
 
         if self.start() is False:
             return False
-        for attachment in mails:
+        for attachment in iter(mails):  # make sure mails are generator to correctly count total_count
             attachment: Attachment
-            e: envelope = attachment.get_envelope()
+            e: Envelope = attachment.get_envelope()
 
             if e.message().strip() == "" or e.subject().strip() == "":
                 # program flow should never allow lead us here
@@ -53,8 +55,8 @@ class MailSender(ABC):
                 print(f"Message for {attachment.mail} already sent, skipping!")
                 continue
 
-            for address in e._to:  # XX this may fail when we put multiple To: addresses in the template header
-                if not validate_email(address):
+            for address in e.to():
+                if not validate_email(address, check_mx=False):
                     logger.error("Erroneous e-mail: {}".format(address))
                     continue
 
@@ -64,9 +66,10 @@ class MailSender(ABC):
                 status = self.process(e)
             except KeyboardInterrupt:
                 status = "interrupted"
-                total_count += len([x for x in mails])
+                total_count += len(list(mails))
                 break
-            except:
+            except Exception as e:
+                logger.error(e)
                 # if something fails (attachment FileNotFoundError, any other error),
                 # we want tag the previously sent e-mails so that they will not be resend next time
                 pass
@@ -90,7 +93,7 @@ class MailSender(ABC):
 
         print("\nSent: {}/{} mails.".format(sent_mails, total_count))
         if sent_mails != total_count:
-            print("Couldn't send all abroad mails. (Details in convey.log.)")
+            print("Could not send all abroad mails. (Details in convey.log.)")
 
 
 class MailSenderOtrs(MailSender):
@@ -103,12 +106,25 @@ class MailSenderOtrs(MailSender):
         Return an appropriate http.client.HTTPResponse object.
         """
 
-        # import ssl; ssl._create_default_https_context = ssl._create_unverified_context  # XX once upon a day (10.2.2017), the certificate stopped working or whatever. This is an ugly solution - i think we may delete this line in few weeks
+        # if not Config.get("otrs_check_ssl", "OTRS", get=bool):
+        # import ssl
+        # ssl._create_default_https_context = ssl._create_unverified_context  # XX once upon a day (10.2.2017), the certificate stopped working or whatever. This is an ugly solution - i think we may delete this line in few weeks
+        # context = ssl._create_unverified_context()
+        # context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
+        # context.options &= ~ssl.OP_NO_TLSv1
 
         content_type, body = self._encode_multipart_formdata(fields, files)
         body = bytes(body, "UTF-8")
-        protocol = host.split(':')[0]
-        h = http.client.HTTPSConnection(host)
+        if "://" not in host:
+            protocol = "https"
+        else:
+            protocol, host = host.split("://", 1)
+        if protocol == "http":
+            h = http.client.HTTPConnection(host)
+        elif protocol == "https":
+            h = http.client.HTTPSConnection(host)
+        else:
+            raise ValueError(f"Unknown protocol in the host {host}")
         if Config.is_debug():
             h.debuglevel = 100
         h.putrequest('POST', selector)
@@ -152,44 +168,43 @@ class MailSenderOtrs(MailSender):
     def _check_record(record, lineno):
         valid = ('CONTACTS' in record)
         if not valid:
-            print(str(sys.stderr) + " Line {}: Record missing CONTACTS field ".format(lineno))
+            print(" Line {}: Record missing CONTACTS field ".format(lineno))
         return valid
 
     @staticmethod
     def _check_response(response):
         response = response.decode("UTF-8")
         if Config.is_testing():
-            logger.info(str(sys.stderr) + " Response length:\n " + str(len(response)))
+            logger.info(" Response length:\n " + str(len(response)))
 
         mo = re_title.search(response)
         if not mo:
-            logger.warning(str(sys.stderr) + " Unrecognized response")
+            logger.warning(" Unrecognized response")
             logger.error(response)
             return False
 
+        if Config.is_debug():
+            logger.info("Got response\n" + response)
+
         title = mo.group(1)
-        if b'P\xc5\x99edat - Tiket -  OTRS'.decode("utf-8") in title or 'Forward - Ticket -  OTRS' in title:
-            # XX r with caron make nb-python fail.
-            # XX we are not sure sending successed. Ex: if we do not include "To" recipient, we land on the same page
+        if 'Předat - Tiket - ' in title or 'Forward - Ticket - ' in title:
+            # XX we are not sure sending succeeded. Ex: if we do not include "To" recipient, we land on the same page
             #   but no message will be send because the page will just state recipient is missing.
-            #   However, this is not a priority, this mostly works.
+            #   However, this is not a priority, this mostly works. You can always use SMTP via --references flag.
             return True
 
         elif title == 'Login - OTRS':
-            logger.warning(str(sys.stderr) + "\n\n *** Not logged in or wrong session cookie ***")
-            return False
-
+            logger.warning("\n\n *** Not logged in or wrong session cookie ***")
         elif title in ('Fatal Error - Frontend -  OTRS', 'Fatal Error - Rozhraní -  OTRS'):
-            logger.warning(str(sys.stderr) + "\n\n *** Bad CSRF token ***")
-            return False
-
+            logger.warning("\n\n *** Bad CSRF token ***")
         else:
-            logger.warning(str(sys.stderr) + " Unrecognized response: " + title)
+            logger.warning(" Unrecognized response: " + title)
             logger.error(response)
-            return False
+        return False
 
-    def ask_value(self, value, description=""):
-        sys.stdout.write('Change {} ({})? [s]kip or paste it: '.format(description, value))
+    @staticmethod
+    def ask_value(value, description=""):
+        sys.stdout.write(f'Change {description} ({value})? [s]kip or paste it: ')
         t = input()
         if not t or t.lower() == "s":
             if not value:
@@ -202,65 +217,77 @@ class MailSenderOtrs(MailSender):
         """ Check and update by dialog OTRS credentials """
         force = False
         while True:
-            if (
-                    force or not self.parser.otrs_id or not self.parser.otrs_num or not self.parser.otrs_cookie or not self.parser.otrs_token or not self.parser.attachment_name):
+            if (force
+                    or not self.parser.otrs_id or not self.parser.otrs_num
+                    or not self.parser.otrs_cookie or not self.parser.otrs_token or not self.parser.attachment_name):
                 self.parser.otrs_id = self.ask_value(self.parser.otrs_id, "ticket url-id")
                 self.parser.otrs_num = self.ask_value(self.parser.otrs_num, "ticket long-num")
                 self.parser.otrs_cookie = self.ask_value(self.parser.otrs_cookie, "cookie")
                 self.parser.otrs_token = self.ask_value(self.parser.otrs_token, "token")
                 self.parser.attachment_name = self.ask_value(self.parser.attachment_name, "attachment name")
-                # if self.parser.attachment_name[-4:] != ".txt":
-                #     self.parser.attachment_name += ".txt"
 
-            sys.stdout.write(
-                "Ticket id = {}, ticket num = {}, cookie = {}, token = {}, attachment_name = {}.\nWas that correct? [y]/n ".format(
-                    self.parser.otrs_id, self.parser.otrs_num, self.parser.otrs_cookie, self.parser.otrs_token,
-                    self.parser.attachment_name))
+            sys.stdout.write(f"Ticket id = {self.parser.otrs_id}, ticket num = {self.parser.otrs_num},"
+                             f" cookie = {self.parser.otrs_cookie}, token = {self.parser.otrs_token},"
+                             f" attachment_name = {self.parser.attachment_name}"
+                             f"\nWas that correct? [y]/n ")
             if input().lower() in ("y", ""):
                 return True
             else:
                 force = True
                 continue
 
-    def process(self, e: envelope):
+    def process(self, e: Envelope):
         def assure_str(c):
-            return c if type(c) is str else ";".join(c)
+            return c if isinstance(c, str) else ";".join(c)
+
+        # body must be HTML because OTRS treats it as such
+        # (we might do a convey option to determine whether OTRS treats a text as plain or html)
+        message = e.message()
+        if not any(x for x in ("<br", "<b>", "<i>", "<p", "<img") if x in message):
+            message = e.message().replace("\n", "<br>\n")
 
         fields = (
             ("Action", "AgentTicketForward"),
             ("Subaction", "SendEmail"),
             ("TicketID", str(self.parser.otrs_id)),
-            ("Email", getaddresses([e._sender])[0][1]),
-            # XX delete this option and rename the other: Config.get("email_from", "SMTP")),
-            ("From", assure_str(e._sender)),  # XX XConfig.get("email_from_name", "SMTP")
-            ("To", assure_str(e._to)),  # mails can be delimited by comma or semicolon
-            ("Subject", e._subject),
-            ("Body", e._message),
+            ("Email", e.from_().address),
+            ("From", assure_str(e.from_())),
+            ("To", assure_str(e.to())),  # mails can be delimited by comma or semicolon
+            ("Subject", e.subject()),
+            ("Body", message),
             ("ArticleTypeID", "1"),  # mail-external
             ("ComposeStateID", "4"),  # open
             ("ChallengeToken", self.parser.otrs_token),
         )
 
-        try:
-            fields += ("SignKeyID", Config.get("signkeyid", "OTRS")),
+        try:  # XX convert "try" to if m := Config.get("signkeyid", "OTRS"):
+            if OTRS_VERSION == 6:
+                fields += ("EmailSecurityOptions", Config.get("signkeyid", "OTRS")),
+            else:
+                fields += ("SignKeyID", Config.get("signkeyid", "OTRS")),
         except KeyError:
             pass
 
-        if e._cc:
-            fields += ("Cc", assure_str(e._cc)),
+        if e.cc():
+            fields += ("Cc", assure_str(e.cc())),
 
-        if e._bcc:  # XX not sure if working
-            fields += ("Bcc", assure_str(e._bcc)),
+        if e.bcc():
+            fields += ("Bcc", assure_str(e.bcc())),
 
         attachment_contents = ""
-        if self.parser.attachment_name and len(e._attachments):
-            attachment_contents = e._attachments[0][0].read_text()
+        if self.parser.attachment_name and len(e.attachments()):
+            attachment_contents = str(e.attachments()[0])
         if attachment_contents:
             files = (("FileUpload", self.parser.attachment_name, attachment_contents),)
         else:
             files = ()
 
-        cookies = (('Cookie', 'Session=%s' % self.parser.otrs_cookie),)
+        # OTRS3 Session
+        # OTRS6 OTRSAgentInterface
+        if OTRS_VERSION == 6:
+            cookies = (('Cookie', f'OTRSAgentInterface={self.parser.otrs_cookie}'),)
+        else:
+            cookies = (('Cookie', 'Session=%s' % self.parser.otrs_cookie),)
 
         if Config.is_testing():
             print(" **** Testing info:")
@@ -277,14 +304,14 @@ class MailSenderOtrs(MailSender):
                                        fields=fields,
                                        files=files,
                                        cookies=cookies)
-        except Exception as e:
+        except Exception:
             import traceback
-            print(traceback.format_exc())
-            print(
-                "\nE-mail couldn't be send to the host {}{} with the fields {}. Are you allowed to send from this e-mail etc?".format(
-                    host, selector, fields))
-            input("Program now ends.")
-            quit()
+            logger.error(traceback.format_exc())
+            logger.error(f"\nE-mail could not be send to the host {host}{selector} with the fields {fields}."
+                  f" Are you allowed to send from this e-mail etc?")
+            # input("Program now ends.")
+            # quit()
+            raise
         if not res or not self._check_response(res.read()):
             print("Sending failure, see convey.log.")
             return False
@@ -305,6 +332,6 @@ class MailSenderSmtp(MailSender):
     def stop(self):
         self.smtp.quit()
 
-    def process(self, e: envelope):
+    def process(self, e: Envelope):
         e.smtp(self.smtp)
         return bool(e.send(True))
